// Copyright 2015 go-swagger maintainers
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Package fileutils exposes utilities to deal with files and paths.
//
// Currently, there is:
//   - [File] to represent an abstraction of an uploaded file.
//     For instance, this is used by [github.com/go-openapi/runtime.File].
//   - path search utilities (e.g. finding packages in the GO search path)
package fileutils
