// Copyright 2015 go-swagger maintainers
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Package easyjson exposes a JSON adapter
// that leverages the [easyjson] serializer library.
//
// It ships as an independent go module.
//
// This library is significantly faster than the standard
// library, provided the data types implement its specific
// interfaces [easyjson.Marshaler] and [easyjson.Unmarshaler].
package easyjson

import (
	_ "github.com/mailru/easyjson" // for documentation purpose only
)
